`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/03/19 00:33:50
// Design Name: 
// Module Name: clock_generator
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

`define HALF_TARGET_FREQ 26'd50000000 
`define TARGET_FREQ_100 19'd500000
module clock_generator(
    clk_1Hz, //divided clock output
    clk_100Hz,
    clk_ctl, //clock for 7-segment
    clk,    //clock input
    rst  
);
    output reg clk_1Hz;
    output reg clk_100Hz; 
    input clk, rst; //clock input
    output reg [1:0] clk_ctl; 
    
    reg [25:0] cnt_1Hz;
    reg [25:0] cnt_1Hz_next;
    reg clk_1Hz_next;
    reg [14:0] cnt_h; //temp buffer for the counter
    reg [8:0] cnt_l; //temp buffer for the counter
    reg [18:0] cnt_100Hz;
    reg [18:0] cnt_100Hz_next;
    reg clk_100Hz_next;

   
    always @*
        begin
            if ({cnt_l, clk_ctl, cnt_h} == `HALF_TARGET_FREQ) //combinational logics to select
                begin
                cnt_1Hz_next = 26'd0;
                clk_1Hz_next = ~clk_1Hz;
                end
            else
                begin
                cnt_1Hz_next = {cnt_l, clk_ctl, cnt_h} + 1'b1;
                clk_1Hz_next = clk_1Hz;
                end
        end
    
    
    //sequential logics
    always @(posedge clk or posedge rst)
    if (rst)
        begin
        {cnt_l, clk_ctl, cnt_h} <= 26'd0;
        clk_1Hz <= 1'b1;
        end
    else
        begin
        {cnt_l, clk_ctl, cnt_h} <= cnt_1Hz_next;
        clk_1Hz <= clk_1Hz_next; 
        end

    always @*
        begin
            if (cnt_100Hz == `TARGET_FREQ_100) //combinational logics to select
                begin
                cnt_100Hz_next = 19'd0;
                clk_100Hz_next = ~clk_100Hz;
                end
            else
                begin
                cnt_100Hz_next = cnt_100Hz + 1'b1;
                clk_100Hz_next = clk_100Hz;
                end
        end

    always @(posedge clk or posedge rst)
    if (rst)
        begin
        cnt_100Hz <= 19'd0;
        clk_100Hz <= 1'b0; ////////1 or 0
        end
    else
        begin
        cnt_100Hz <= cnt_100Hz_next;
        clk_100Hz <= clk_100Hz_next; 
        end
        
    
endmodule
