`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2019/04/02 16:30:11
// Design Name: 
// Module Name: topmodule
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`define ENABLE 1'b1
`define LIGHT_0 16'b0000_0000_0000_0000
module topmodule(
    D_ssd,
    ssd_ctl,
    LED_out,
    clk,
    pb_0,
    pb_1,
    pb_2
    );

    output reg [7:0] D_ssd;
    output reg [3:0] ssd_ctl;
    output reg [15:0] LED_out;
    input clk;
    input pb_0, pb_1, pb_2; 
    
    wire clk_1Hz;
    wire clk_100Hz;
    wire [1:0] clk_ssd_ctl;

    wire rst;

    wire pb_0_debounced, pb_1_debounced, pb_2_debounced;
    wire pb_0_time_set, pb_0_select_mode;
    wire pb_0_out_pulse, pb_1_out_pulse, pb_2_out_pulse;

    wire pb_2_normal, pb_2_long;
    
    wire mode_result;
    wire setting;

    wire [7:0] D_ssd_a, D_ssd_b;
    wire [3:0] ssd_ctl_a, ssd_ctl_b;

    wire [15:0] LED_out_tmp;
   

    clock_generator U0(
    .clk_1Hz(clk_1Hz), //divided clock output
    .clk_100Hz(clk_100Hz),
    .clk_ctl(clk_ssd_ctl), //clock for 7-segment
    .clk(clk),    //clock input
    .rst(rst)   //low-active reset
    );

    debounce U1( 
    .rst(rst), 
    .clk(clk_100Hz), 
    .pb_in(pb_0),
    .pb_debounced(pb_0_debounced)
    );

    press_long U2( 
    .pb_press_long(pb_0_time_set), //short press output
    .rst_count(pb_0_select_mode),  //long press output
    .pb_debounce_in(pb_0_debounced),
    .clk(clk)
);
    one_pulse U3( 
    .rst(rst), 
    .clk(clk), 
    .in_trig(pb_0_time_set),
    .out_pulse(pb_0_out_pulse)
);

    debounce U4( 
    .rst(rst), 
    .clk(clk_100Hz), 
    .pb_in(pb_1),
    .pb_debounced(pb_1_debounced)
    );

    one_pulse U5( 
    .rst(rst), 
    .clk(clk), 
    .in_trig(pb_1_debounced),
    .out_pulse(pb_1_out_pulse)
);

    debounce U6( 
    .rst(rst), 
    .clk(clk_100Hz), 
    .pb_in(pb_2),
    .pb_debounced(pb_2_debounced)
    );

    press_long U7( 
    .pb_press_long(pb_2_normal), //short press normal function
    .rst_count(pb_2_long),  //long press output
    .pb_debounce_in(pb_2_debounced),
    .clk(clk)
);

    one_pulse U8( 
    .rst(rst), 
    .clk(clk), 
    .in_trig(pb_2_normal),
    .out_pulse(pb_2_out_pulse)
);

    fsm_mode_change U9(
    .select_mode_change(pb_0_select_mode),
    .clk(clk),
    .rst(rst),
    .mode_result(mode_result)
);

    fsm_setting U10(
    .select_mode_change(pb_0_out_pulse),
    .clk(clk),
    .rst(rst),
    .setting(setting)
);

    lab_7_1_top U11(
    .D_ssd(D_ssd_a),
    .ssd_ctl(ssd_ctl_a),
    .clk(clk),
    .clk_1Hz(clk_1Hz),
    .clk_100Hz(clk_100Hz),
    .clk_ssd_ctl(clk_ssd_ctl),
    .rst(pb_2_long),
    .pb_count_pause(pb_1_out_pulse),
    .pb_lap(pb_2_out_pulse)
    );

    lab_7_2_top U12(
    .D_ssd(D_ssd_b),
    .ssd_ctl(ssd_ctl_b),
    .LED_out(LED_out_tmp),
    .clk(clk),
    .clk_1Hz(clk_1Hz),
    .clk_100Hz(clk_100Hz),
    .clk_ssd_ctl(clk_ssd_ctl),
    .pb_pause(pb_1_out_pulse),
    .pb_stop(pb_2_out_pulse),
    .pb_hour(pb_1_out_pulse),
    .pb_min(pb_2_out_pulse),
    .time_set(setting),
    .rst(rst),
    .stop_to_zero(pb_2_long)
    );

    always @*
        if (mode_result)
            begin
                D_ssd = D_ssd_b;
                ssd_ctl = ssd_ctl_b;
                LED_out = LED_out_tmp;
            end
        else
            begin
                D_ssd = D_ssd_a;
                ssd_ctl = ssd_ctl_a;
                LED_out = `LIGHT_0;
            end


   
endmodule
